# $Id: quads-b_n.gap,v 1.1 2006/01/15 12:29:26 maxhorn Exp $
#
# This GAP program attempts to iterate over all "special" quadrangles as 
# described in 6.3.16 and 6.3.17. For each such quadrangle it then computes
# the subspace 'l' and tries to find a geometric point in it.
#
# To make it at least remotely feasible to iterate over all the quadrangles,
# we use all sorts of tricks to reduce them to certain special cases.
# Still, only relatively small values of q are doable for now... 
#

# The parameters of our group. Works for any prime power q>=4, but n must be 3.
# (In fact, the value of n is not used anywhere).
n := 3;
q := 9;

#
# Various usefil bits and pieces
#
F := GF(q*q);
z := PrimitiveRoot(F);
elem := Elements(F);

ProdA := function(u, v)
	local n, i, sum;
	n := (Length(u)-1)/2;
	sum := u[2*n+1] * v[2*n+1];;
	for i in [1..n] do
		sum := sum + u[i] * v[i+n];
		sum := sum + u[i+n] * v[i];
	od;
	return sum;
	#return Sum([1..n], i -> u[i] * v[i+n]) - Sum([1..n], i -> u[i+n] * v[i]);
	#return u*B*v;
end;;

ProdB := function(u, v)
	local n, i, sum;
	n := Length(u);
	sum := 0;
	for i in [1..n] do
		sum := sum + u[i] * v[i]^q;
	od;
	return sum;
end;;

NormB := function(u)
	return ProdB(u,u);
end;;

# The set of units in F_{q^2}
FStar := Filtered(F, g -> g <> 0*z);

# Computer the outer produce of 'space' with itself using the two-argument 
# function 'func'. We use this to compute the gram matrices of a given
# vector span.
Outer := function (space, func)
	local mat, n, x, y;
	n := Length(space);
	mat := NullMat(n, n);
	for x in [1..n] do
		for y in [1..n] do
			mat[x][y] := func(space[x], space[y]);
		od;
	od;
	return mat;
end;;

# Determine whether the vectors in 'space' span a geometric subspace.
# Condition for that is that the subspace is non-degenerate w.r.t. the
# first form (ProdA) and totally degenerate w.r.t the second form (ProdB).
IsGeometric := function (space)
	local n;
	n := Length(space);
	if (n > 1) and (Outer(space, ProdA) <> NullMat(n, n, F)) then
		return false;
	fi;
	return 0*z <> Determinant(Outer(space, ProdB));
end;;


# Check whether the given quadrangle is geometric
IsQuadrangle := function (a, b, c, d)
	return IsGeometric([a, b])
		and IsGeometric([b, c])
		and IsGeometric([c, d])
		and IsGeometric([d, a])
		and IsGeometric([a])
		and IsGeometric([b])
		and IsGeometric([c])
		and IsGeometric([d])
		;
end;;

IsSplitQuadrangle := function (a, b, c, d, p)
	return IsGeometric([p, a])
		and IsGeometric([p, b])
		and IsGeometric([p, c])
		and IsGeometric([p, d])
		;
end;;

# Check whether the given quadrangle is a special one
IsDifficultQuadrangle := function (a, b, c, d)
	return 0*z <> ProdA(a, c)
		and 0*z <> ProdA(b, d)
		;
end;;

IsPointSplitting := function(a, b, c, d, p2, p3, p6, p7)
	local p1, p;
	p1 := -(c[6]*p3 + c[3]*p6 + c[7]*p7) + p2 * b[1];
	p := [p1, p2, p3, 0, 0, p6, p7] * z^0;
 	return IsGeometric([p]) and IsSplitQuadrangle(a, b, c, d, p);
end;;

# Given a triangle, try to "split"
TryToSplitQuad := function (a, b, c, d)
	local p6, p7;
	# We need a geometric element which connects a,b,c,d.

	# p3 = 0  => p7 = 0, and wlog p6 = 1
	if IsPointSplitting(a, b, c, d, -d[3], 0, 1, 0) then
		return true;
	fi;

	# p3 = 1 => p6 = -p7^2/2
	for p7 in F do
		p6 := -p7*p7/2;
		if (IsPointSplitting(a, b, c, d, -(d[6] + d[3]*p6 + d[7]*p7), 1, p6, p7)) then
			return true;
		fi;
	od;
	Display("Konnte das Quad nicht splitten!");
	Display([a, b, c, d]);
	#Assert(0, false);
	return false;
end;;

ExtTryToSplitQuad := function (a, b, c, d)
	local p1, p2, p3, p6, p7, p;
	# We need a geometric element which connects a,b,c,d.

	for p2 in F do
		# p3 = 0  => p7 = 0, and wlog p6 = 1
		p3 := 0;
		p6 := 1;
		p7 := 0;
		p1 := -c[3] + p2 * b[1];
		p := [p1, p2, p3, 0, 0, p6, p7] * z^0;
		if IsGeometric([p]) and IsGeometric([p,a]) and IsGeometric([p,b]) and IsGeometric([p,c]) then
				FIXME: We can't just call TryToSplitQuad for a,p,c,d, since p is of the wrong form
			if TryToSplitQuad(a, p, c, d) then
				Display("Splitting successful!");
				Display(p);
				return true;
			fi;
		fi;
	
		# p3 = 1 => p6 = -p7^2/2
		p3 := 1;
		for p7 in F do
			p6 := -p7*p7/2;
			p1 := -(c[6]*p3 + c[3]*p6 + c[7]*p7) + p2 * b[1];
			p := [p1, p2, p3, 0, 0, p6, p7] * z^0;
			if IsGeometric([p]) and IsGeometric([p,a]) and IsGeometric([p,b]) and IsGeometric([p,c]) then
			
				FIXME: We can't just call TryToSplitQuad for a,p,c,d, since p is of the wrong form
				if TryToSplitQuad(a, p, c, d) then
					Display("Splitting successful!");
					Display(p);
					return true;
				fi;
			fi;
		od;
	od;
	Display("Konnte das Quad nicht splitten!");
	Display([a, b, c, d]);
	#Assert(0, false);
	return false;
end;;


for p1 in points do
	for p2 in points do
		if not IsGeometric([p1,p2]) then
			continue;
		fi;
		Display("bar");
		if IsGeometric([p1,a]) and IsGeometric([p1,b]) and IsGeometric([c,p2]) and IsGeometric([d,p2]) then
			Display("WOW");
		fi;
		if IsGeometric([p1,c]) and IsGeometric([p1,d]) and IsGeometric([a,p2]) and IsGeometric([b,p2]) then
			Display("WOW");
		fi;
		if IsGeometric([p1,c]) and IsGeometric([p1,b]) and IsGeometric([a,p2]) and IsGeometric([d,p2]) then
			Display("WOW");
		fi;
	od;
od;


# Iterate over all the quadrangles, and call TryToSplitQuad for every non-trivially split
# special quad.
IterateQuadrangles := function ()
	local a, b, c, d, b1, c2, c3, c6, c7, d3, d6, d7, i, j;
	i := 0;
	j := 0;
	a := [1, 0, 0, 0, 0, 0, 0] * z^0;
	for b1 in [z^0, z^1] do
		b := [b1, 1, 0, 0, 0, 0, 0] * z^0;
		if not IsGeometric([b]) then
			continue;
		fi;
		c := [0, 0, 0, 1, -b1, 0, 0] * z^0;
		for c2 in elem do
			c[2] := c2;
			for c3 in elem do
				c[3] := c3;
				for c6 in elem do
					c[6] := c6;
					for c7 in elem do
						c[7] := c7;
						c[1] := c2 * b1 - c3*c6 - c7*c7/2;
						
						if not IsGeometric([c])
						      or not IsGeometric([b, c])
						      or ProdA(a, c) = 0*z then
							continue;
						fi;
						
						i := i + 1;
						
						d := [0, 0, 0, 0, 1, 0, 0] * z^0;
						for d3 in elem do
							d[3] := d3;
							for d6 in elem do
								d[6] := d6;
								for d7 in elem do
									d[7] := d7;
									
									d[2] := -(d3*d6 + d7*d7/2);
									d[1] := -(c2 + c6*d3 + c3*d6 + c7*d7 - b1 * d[2]);
									
									if IsGeometric([d])
									  and IsGeometric([c, d])
									  and IsGeometric([d, a])
									  and 0*z <> ProdA(b, d)
									  then
										TryToSplitQuad(a, b, c, d);
										j := j + 1;
									fi;
								od;
							od;
								Print("i = ", i, ", j = ", j, "\n");
						od;
					od;
				od;
			od;
		od;
		Print("i = ", i, ", j = ", j, "\n");
	od;
end;;
