# $Id: tris.gap,v 1.5 2006/02/04 13:15:56 maxhorn Exp $

#
# Various useful bits and pieces: The underlying field etc.
#
q := 5;
F := GF(q*q);
z := PrimitiveRoot(F);
elem := Elements(F);

# The set of units in F_{q^2}
FStar := Filtered(F, g -> g <> 0*z);

# The product w.r.t. the first form (.,.), i.e. the f-form
ProdA := function(u, v)
	local n, i, sum;
	n := (Length(u)-1)/2;
	sum := u[2*n+1] * v[2*n+1];;
	for i in [1..n] do
		sum := sum + u[i] * v[i+n];
		sum := sum + u[i+n] * v[i];
	od;
	return sum;
end;;

# The product w.r.t. the second form ((.,.)), i.e. the g-form
ProdB := function(u, v)
	local n, i, sum;
	n := Length(u);
	sum := 0;
	for i in [1..n] do
		sum := sum + u[i] * v[i]^q;
	od;
	return sum;
end;;

# Computer the outer produce of 'space' with itself using the two-argument 
# function 'func'. We use this to compute the gram matrices of a given
# vector span.
Outer := function (space, func)
	local mat, n, x, y;
	n := Length(space);
	mat := NullMat(n, n);
	for x in [1..n] do
		for y in [1..n] do
			mat[x][y] := func(space[x], space[y]);
		od;
	od;
	return mat;
end;;

# Determine whether the vectors in 'space' span a geometric subspace.
# Condition for that is that the subspace is non-degenerate w.r.t. the
# first form (ProdA) and totally degenerate w.r.t the second form (ProdB).
IsGeometric := function (space)
	local n;
	n := Length(space);
	if (n > 1) and (Outer(space, ProdA) <> NullMat(n, n, F)) then
		return false;
	fi;
	return 0*z <> Determinant(Outer(space, ProdB));
end;;

# Determine whether the given three vertices form a triangle. That is,
# whether each of them forms a point of the geometry, and whether
# the subspaces they span pairwise are lines of the geometry.
IsTriangle := function (a, b, c)
	return IsGeometric([a, b])
		and IsGeometric([b, c])
		and IsGeometric([c, a])
		and IsGeometric([a])
		and IsGeometric([b])
		and IsGeometric([c])
		;
end;;

# Determine whether the given three vertices form a geometric triangle,
# i.e. a triangle (see IsTriangle) where the three vertices span a plane
# of the geometry.
IsGeoTriangle := function (a, b, c)
	return IsGeometric([a, b, c])
		and IsTriangle(a, b, c)
		;
end;;

# Try to decompose the triangle using the octahedron construction.
# Note: We expect a,b,c to be of the form e1,e2,v3 as described in the
# paper.
# This function will never incorrectly report a triangle as decomposable which
# really is decomposable, but may fail to decompose a triangle that is
# actually decomposable.
TryToDecomposeTri := function (a, b, c)
	local p1, p2, p3,
		  sigma, theta,
		  alpha, beta, gamma, epsilon;
	
	alpha := c[1];
	beta := c[2];
	gamma := c[3];
	epsilon := c[7];
	
	p1 := [0, 0, 0, 0, 0, 1, 0] * z^0;
	p2 := [0, 0, 0, 0, -1/beta, 1/gamma, 0] * z^0;
	p3 := [0, 0, 0, -1/alpha, 0, 1/gamma, 0] * z^0;
	
	# We already know that [p1], [p1,a], [p1,b], [p1,a,b] are geometric,
	# so we won't bother to check this in the following.
	for sigma in F do
		p3[1] := sigma^2*alpha/2;
		p3[4] := -1/alpha;
		p3[6] := (1 - sigma*epsilon) / gamma;
		p3[7] := sigma;

		# Short circuit: If p3 or any of the lines/triangles extending from
		# it are not geometric we can immediately skip this sigma value.
		if not IsGeometric([p3]) 
			or not IsGeometric([p3, p1])
			or not IsGeometric([p3, c])
			or not IsGeometric([p3, b])
			or not IsGeometric([p3, b, c])
			or not IsGeometric([p3, b, p1])
			then
			continue;
		fi;

		for theta in F do
			p2[1] := theta * sigma * alpha;
			p2[2] := theta^2*beta/2;
			p2[5] := -1/beta;
			p2[6] := (1 - theta*epsilon) / gamma;
			p2[7] := theta;

			if IsGeometric([p2])
				and IsGeometric([p2,p1])
				and IsGeometric([p2,a])
				and IsGeometric([p2,c])
				and IsGeometric([p2,p3]) 
				and IsGeometric([p1,p2,p3]) 
				and IsGeometric([a,p1,p2]) 
				and IsGeometric([c,p2,p3]) 
				and IsGeometric([p2,a,c]) 
				then
				Print("\n");
				return [p1,p2,p3];
			fi;
			Print(".\c");
		od; # end theta
	od; # end sigma
	Print("\n");
	return [];
end;;


a := [1, 0, 0, 0, 0, 0, 0] * z^0;
b := [0, 1, 0, 0, 0, 0, 0] * z^0;

# We fix epsilon to be 1, and determine suitable matching values
# for delta 1 and gamma.
epsilon := z^0;
for delta in FStar do
	gamma := -epsilon*epsilon/(2*delta);
	if gamma^(q+1)+delta^(q+1)+epsilon^(q+1) = 0*z^0 then
		break;
	fi;
od;

# It's sufficient to perform all computations with the single triple
# we just determined, due to the transitivity of the stabilizer of e1/e2
# on the one-dimensional spaces which are singular w.r.t to both forms
# (see also Lemma 3.1'1 proof).


# Iterate over all possible pairs (alpha, beta) and try to
# decompose the corresponding triangle (only if the result
# actually *is* a triangle).
i := 0;
j := 0;
failures := 0;
for alpha in FStar do
	for beta in FStar do
		c := [alpha, beta, gamma, 0, 0, delta, epsilon ] * z^0;	# q = 5
		i := i + 1;
		if IsTriangle(a, b, c) then
			j := j + 1;
			Print("Decomposing triangle (",i,",",j,"):\n");
			res := TryToDecomposeTri(a, b, c);
			if res = [] then
				Display(" ... FAILED");
				failures := failures + 1;
			else
				Display(" ... SUCCESS");
			fi;
		fi;
	od;
od;
Print("failures = ", failures, "\n");
